@@ -1,12 +1,15 @@
 __author__ = 'Oleg Butovich'
-__copyright__ = '(c) Oleg Butovich 2013'
+__copyright__ = '(c) Oleg Butovich 2013-2017'
 __licence__ = 'MIT'
 
 
 import json
+import sys
 
 try:
     import requests
+    urllib3 = requests.packages.urllib3
+    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
     from requests.auth import AuthBase
     from requests.cookies import cookiejar_from_dict
 except ImportError:
@@ -15,11 +18,32 @@ except ImportError:
     sys.exit(1)
 
 
+if sys.version_info[0] >= 3:
+    import io
+    def is_file(obj): return isinstance(obj, io.IOBase)
+else:
+    def is_file(obj): return isinstance(obj, file)
+
+
+class AuthenticationError(Exception):
+    def __init__(self, msg):
+        super(AuthenticationError, self).__init__(msg)
+        self.msg = msg
+
+    def __str__(self):
+        return self.msg
+
+    def __repr__(self):
+        return self.__str__()
+
+
 class ProxmoxHTTPAuth(AuthBase):
-    def __init__(self, base_url, username, password):
+    def __init__(self, base_url, username, password, verify_ssl=False):
         response_data = requests.post(base_url + "/access/ticket",
-                                      verify=False,
+                                      verify=verify_ssl,
                                       data={"username": username, "password": password}).json()["data"]
+        if response_data is None:
+            raise AuthenticationError("Couldn't authenticate user: {0} to {1}".format(username, base_url + "/access/ticket"))
 
         self.pve_auth_cookie = response_data["ticket"]
         self.csrf_prevention_token = response_data["CSRFPreventionToken"]
@@ -29,6 +53,17 @@ class ProxmoxHTTPAuth(AuthBase):
         return r
 
 
+class ProxmoxHTTPTokenAuth(ProxmoxHTTPAuth):
+    """Use existing ticket/token to create a session.
+
+    Overrides ProxmoxHTTPAuth so that an existing auth cookie and csrf token
+    may be used instead of passing username/password.
+    """
+    def __init__(self, auth_token, csrf_token):
+        self.pve_auth_cookie = auth_token
+        self.csrf_prevention_token = csrf_token
+
+
 class JsonSerializer(object):
 
     content_types = [
@@ -36,16 +71,16 @@ class JsonSerializer(object):
         "application/x-javascript",
         "text/javascript",
         "text/x-javascript",
-        "text/x-json",
-    ]
+        "text/x-json"
+        ]
 
     def get_accept_types(self):
-        return self.content_types
+        return ", ".join(self.content_types)
 
     def loads(self, response):
         try:
-            return json.loads(response.content)['data']
-        except ValueError:
+            return json.loads(response.content.decode('utf-8'))['data']
+        except (UnicodeDecodeError, ValueError):
             return response.content
 
 
@@ -58,8 +93,8 @@ class ProxmoxHttpSession(requests.Session):
         #filter out streams
         files = files or {}
         data = data or {}
-        for k, v in data.copy().iteritems():
-            if isinstance(v, file):
+        for k, v in data.copy().items():
+            if is_file(v):
                 files[k] = v
                 del data[k]
 
@@ -72,9 +107,13 @@ class ProxmoxHttpSession(requests.Session):
 
 
 class Backend(object):
-    def __init__(self, host, user, password, port=8006, verify_ssl=True, mode='json', timeout=5):
+    def __init__(self, host, user, password, port=8006, verify_ssl=True,
+                 mode='json', timeout=5, auth_token=None, csrf_token=None):
         self.base_url = "https://{0}:{1}/api2/{2}".format(host, port, mode)
-        self.auth = ProxmoxHTTPAuth(self.base_url, user, password)
+        if auth_token is not None:
+            self.auth = ProxmoxHTTPTokenAuth(auth_token, csrf_token)
+        else:
+            self.auth = ProxmoxHTTPAuth(self.base_url, user, password, verify_ssl)
         self.verify_ssl = verify_ssl
         self.mode = mode
         self.timeout = timeout
@@ -94,3 +133,7 @@ class Backend(object):
     def get_serializer(self):
         assert self.mode == 'json'
         return JsonSerializer()
+
+    def get_tokens(self):
+        """Return the in-use auth and csrf tokens."""
+        return self.auth.pve_auth_cookie, self.auth.csrf_prevention_token
