@@ -1,5 +1,5 @@
 __author__ = 'Oleg Butovich'
-__copyright__ = '(c) Oleg Butovich 2013'
+__copyright__ = '(c) Oleg Butovich 2013-2017'
 __licence__ = 'MIT'
 
 
@@ -20,13 +20,15 @@ class ProxmoxParamikoSession(ProxmoxBaseSSHSession):
                  password=None,
                  private_key_file=None,
                  port=22,
-                 timeout=5):
+                 timeout=5,
+                 sudo=False):
         self.host = host
         self.username = username
         self.password = password
         self.private_key_file = private_key_file
         self.port = port
         self.timeout = timeout
+        self.sudo = sudo
         self.ssh_client = self._connect()
 
     def _connect(self):
@@ -50,6 +52,8 @@ class ProxmoxParamikoSession(ProxmoxBaseSSHSession):
         return ssh_client
 
     def _exec(self, cmd):
+        if self.sudo:
+            cmd = 'sudo ' + cmd
         session = self.ssh_client.get_transport().open_session()
         session.exec_command(cmd)
         stdout = ''.join(session.makefile('rb', -1))
@@ -58,17 +62,17 @@ class ProxmoxParamikoSession(ProxmoxBaseSSHSession):
 
     def upload_file_obj(self, file_obj, remote_path):
         sftp = self.ssh_client.open_sftp()
-        remote_file = sftp.open(remote_path, 'wb')
-        remote_file.write(file_obj.read())
-        remote_file.close()
+        sftp.putfo(file_obj, remote_path)
+        sftp.close()
 
 
 class Backend(BaseBackend):
-    def __init__(self, host, user, password=None, private_key_file=None, port=22, timeout=5):
+    def __init__(self, host, user, password=None, private_key_file=None, port=22, timeout=5, sudo=False):
         self.session = ProxmoxParamikoSession(host, user,
                                               password=password,
                                               private_key_file=private_key_file,
                                               port=port,
-                                              timeout=timeout)
+                                              timeout=timeout,
+                                              sudo=sudo)
 
 
